# !/usr/bin/env python


DASH = '-' * 50


def sample_yaml_file():
    return "project: Sample Project" + "\n" + \
           "commands:" + "\n" + \
           "  - build: ./gradlew clean build" + "\n" + \
           "  - lint: ./gradlew spotlessApply"


def help_message(project):
    return "Usage: 1build <command_name> \n\n" + project.__str__()



#!/usr/bin/env python

import os

from .utils import DASH


def execute(command, before=None, after=None):
    cmd = command.cmd
    print(DASH + "\nName: " + command.name + "\nCommand: " + command.cmd)
    if before:
        print("Before: " + before)
        cmd = before + " && " + cmd
    if after:
        print("After: " + after)
        cmd = cmd + " && " + after
    print(DASH)
    os.system(cmd)



#!/usr/bin/env python

import os

from ruamel.yaml import YAML

from .project import Command, Project
from .utils import DASH, sample_yaml_file


def parse_project_config(build_file_name):
    if os.path.exists(build_file_name):
        with open(build_file_name, 'r') as stream:
            try:
                yaml = YAML(typ="safe")
                content = yaml.load(stream)
                before = content.get("before", None)
                after = content.get("after", None)
                return Project(
                    name=(content["project"]),
                    before=before,
                    after=after,
                    commands=__command_list__(content["commands"])
                )
            except Exception:
                raise ValueError(__parsing_error_message__(build_file_name))
    else:
        raise ValueError(__file_not_found_error_message__(build_file_name))


def __command_list__(raw_string):
    commands = []
    for cmd in raw_string:
        for key, val in cmd.items():
            commands.append(Command(name=key, cmd=val))
    return commands


def __file_not_found_error_message__(build_file_name):
    return "No '" + build_file_name + "' file found in current directory."


def __parsing_error_message__(build_file_name):
    return "Error in parsing '" + build_file_name + "' config file." + \
           " Make sure file is in correct format." + \
           "\nSample format is:\n\n" + DASH + "\n" + sample_yaml_file() + \
           "\n" + DASH + "\n"



#!/usr/bin/env python

import sys

from .config_parser import parse_project_config
from .executor import execute
from .input_parser import command_to_run
from .utils import help_message

BUILD_FILE_NAME = "1build.yaml"


def run(build_file_name, arguments):
    global BUILD_FILE_NAME
    BUILD_FILE_NAME = build_file_name
    try:
        project = parse_project_config(build_file_name)
        command_name = command_to_run(arguments)
        if command_name == "help":
            print(help_message(project))
        else:
            command = project.get_command(command_name)
            execute(command, project.before, project.after)
    except ValueError as error:
        print(error)


def main():
    run(BUILD_FILE_NAME, sys.argv)



# !/usr/bin/env python

from .utils import help_message


class Command:
    def __init__(self, name, cmd):
        self.name = name
        self.cmd = cmd

    def __str__(self):
        return self.name + " | " + self.cmd


class Project:
    def __init__(self, name, before, after, commands):
        self.name = name
        self.before = before
        self.after = after
        self.commands = commands

    def get_command(self, command_name):
        if self.__has_command__(command_name):
            for cmd in self.commands:
                if cmd.name == command_name:
                    return cmd
        else:
            raise ValueError(
                "No command '" + command_name + "' found in config file" +
                "\n\n" + help_message(self)
            )

    def __has_command__(self, command_name):
        for cmd in self.commands:
            if cmd.name == command_name:
                return True
        return False

    def available_commands(self):
        return "\n".join(map(str, self.commands))

    def __str__(self):
        return "project: " + self.name + "\ncommands:\n" + \
               self.available_commands()
